@@ -17,7 +17,7 @@ from ruamel.yaml.compat import utf8, text_type, PY2, nprint, dbg, DBG_EVENT, \
     check_anchorname_char
 
 if False:  # MYPY
-    from typing import Any, Dict, List, Union, Text  # NOQA
+    from typing import Any, Dict, List, Union, Text, Tuple  # NOQA
     from ruamel.yaml.compat import StreamType  # NOQA
 
 __all__ = ['Emitter', 'EmitterError']
@@ -43,6 +43,43 @@ class ScalarAnalysis(object):
         self.allow_block = allow_block
 
 
+class Indents(object):
+    # replacement for the list based stack of None/int
+    def __init__(self):
+        # type: () -> None
+        self.values = []  # type: List[Tuple[int, bool]]
+
+    def append(self, val, seq):
+        # type: (Any, Any) -> None
+        self.values.append((val, seq))
+
+    def pop(self):
+        # type: () -> Any
+        return self.values.pop()[0]
+
+    def last_seq(self):
+        # type: () -> bool
+        # return the seq(uence) value for the element added before the last one
+        # in increase_indent()
+        try:
+            return self.values[-2][1]
+        except IndexError:
+            return False
+
+    def seq_flow_align(self, seq_indent, column):
+        # type: (int, int) -> int
+        # extra spaces because of dash
+        if len(self.values) < 2 or not self.values[-1][1]:
+            return 0
+        # -1 for the dash
+        base = self.values[-1][0] if self.values[-1][0] is not None else 0
+        return base + seq_indent - column - 1
+
+    def __len__(self):
+        # type: () -> int
+        return len(self.values)
+
+
 class Emitter(object):
     DEFAULT_TAG_PREFIXES = {
         u'!': u'!',
@@ -74,7 +111,7 @@ class Emitter(object):
         self.event = None  # type: Any
 
         # The current indentation level and the stack of previous indents.
-        self.indents = []  # type: List[Union[None, int]]
+        self.indents = Indents()
         self.indent = None  # type: Union[None, int]
 
         # Flow level.
@@ -109,16 +146,17 @@ class Emitter(object):
         self.allow_unicode = allow_unicode
         # set to False to get "\Uxxxxxxxx" for non-basic unicode like emojis
         self.unicode_supplementary = sys.maxunicode > 0xffff
-        self.block_seq_indent = block_seq_indent if block_seq_indent else 0
+        self.sequence_dash_offset = block_seq_indent if block_seq_indent else 0
         self.top_level_colon_align = top_level_colon_align
-        self.best_indent = 2
+        self.best_sequence_indent = 2
         self.requested_indent = indent  # specific for literal zero indent
         if indent and 1 < indent < 10:
-            self.best_indent = indent
-        # if self.best_indent < self.block_seq_indent + 1:
-        #     self.best_indent = self.block_seq_indent + 1
+            self.best_sequence_indent = indent
+        self.best_map_indent = self.best_sequence_indent
+        # if self.best_sequence_indent < self.sequence_dash_offset + 1:
+        #     self.best_sequence_indent = self.sequence_dash_offset + 1
         self.best_width = 80
-        if width and width > self.best_indent * 2:
+        if width and width > self.best_sequence_indent * 2:
             self.best_width = width
         self.best_line_break = u'\n'  # type: Any
         if line_break in [u'\r', u'\n', u'\r\n']:
@@ -152,6 +190,16 @@ class Emitter(object):
             raise YAMLStreamError('stream argument needs to have a write() method')
         self._stream = val
 
+    @property
+    def serializer(self):
+        # type: () -> Any
+        try:
+            if hasattr(self.dumper, 'typ'):
+                return self.dumper.serializer  # type: ignore
+            return self.dumper._serializer  # type: ignore
+        except AttributeError:
+            return self  # cyaml
+
     def dispose(self):
         # type: () -> None
         # Reset the state attributes (to clear self-references)
@@ -200,16 +248,25 @@ class Emitter(object):
 
     def increase_indent(self, flow=False, sequence=None, indentless=False):
         # type: (bool, bool, bool) -> None
-        self.indents.append(self.indent)
-        if self.indent is None:
+        self.indents.append(self.indent, sequence)
+        if self.indent is None:  # top level
             if flow:
-                self.indent = self.best_indent
+                # self.indent = self.best_sequence_indent if self.indents.last_seq() else \
+                #              self.best_map_indent
+                # self.indent = self.best_sequence_indent
+                self.indent = self.requested_indent
             else:
                 self.indent = 0
         elif not indentless:
-            self.indent += self.best_indent
-            # if self.sequence_context and (self.block_seq_indent + 2) > self.best_indent:
-            #    self.indent = self.block_seq_indent + 2
+            self.indent += (self.best_sequence_indent if self.indents.last_seq() else
+                            self.best_map_indent)
+            # if self.indents.last_seq():
+            #     if self.indent == 0: # top level block sequence
+            #         self.indent = self.best_sequence_indent - self.sequence_dash_offset
+            #     else:
+            #         self.indent += self.best_sequence_indent
+            # else:
+            #     self.indent += self.best_map_indent
 
     # States.
 
@@ -361,9 +418,10 @@ class Emitter(object):
 
     def expect_flow_sequence(self):
         # type: () -> None
-        self.write_indicator(u'[', True, whitespace=True)
-        self.flow_level += 1
+        ind = self.indents.seq_flow_align(self.best_sequence_indent, self.column)
+        self.write_indicator(u' ' * ind + u'[', True, whitespace=True)
         self.increase_indent(flow=True, sequence=True)
+        self.flow_level += 1
         self.state = self.expect_first_flow_sequence_item
 
     def expect_first_flow_sequence_item(self):
@@ -410,7 +468,8 @@ class Emitter(object):
 
     def expect_flow_mapping(self):
         # type: () -> None
-        self.write_indicator(u'{', True, whitespace=True)
+        ind = self.indents.seq_flow_align(self.best_sequence_indent, self.column)
+        self.write_indicator(u' ' * ind + u'{', True, whitespace=True)
         self.flow_level += 1
         self.increase_indent(flow=True, sequence=False)
         self.state = self.expect_first_flow_mapping_key
@@ -502,8 +561,9 @@ class Emitter(object):
                 self.write_pre_comment(self.event)
             nonl = self.no_newline if self.column == 0 else False
             self.write_indent()
-            self.write_indicator((u' ' * self.block_seq_indent) + u'-', True, indention=True)
-            if nonl or self.block_seq_indent + 2 > self.best_indent:
+            ind = self.sequence_dash_offset  # if  len(self.indents) > 1 else 0
+            self.write_indicator(u' ' * ind + u'-', True, indention=True)
+            if nonl or self.sequence_dash_offset + 2 > self.best_sequence_indent:
                 self.no_newline = True
             self.states.append(self.expect_block_sequence_item)
             self.expect_node(sequence=True)
@@ -850,8 +910,9 @@ class Emitter(object):
                 if ch in u'#,[]{}&*!|>\'\"%@`':
                     flow_indicators = True
                     block_indicators = True
-                if ch in u'?:':
-                    flow_indicators = True
+                if ch in u'?:':  # ToDo
+                    if self.serializer.use_version == (1, 1):
+                        flow_indicators = True
                     if followed_by_whitespace:
                         block_indicators = True
                 if ch == u'-' and followed_by_whitespace:
@@ -859,7 +920,9 @@ class Emitter(object):
                     block_indicators = True
             else:
                 # Some indicators cannot appear within a scalar as well.
-                if ch in u',?[]{}':
+                if ch in u',[]{}':  # http://yaml.org/spec/1.2/spec.html#id2788859
+                    flow_indicators = True
+                if ch == u'?' and self.serializer.use_version == (1, 1):
                     flow_indicators = True
                 if ch == u':':
                     if followed_by_whitespace:
@@ -1184,7 +1247,7 @@ class Emitter(object):
         hints = u''
         if text:
             if text[0] in u' \n\x85\u2028\u2029':
-                hints += text_type(self.best_indent)
+                hints += text_type(self.best_sequence_indent)
             if text[-1] not in u'\n\x85\u2028\u2029':
                 hints += u'-'
             elif len(text) == 1 or text[-2] in u'\n\x85\u2028\u2029':
@@ -1388,14 +1451,15 @@ class Emitter(object):
         if comments is None:
             return False
         try:
+            start_events = (MappingStartEvent, SequenceStartEvent)
             for comment in comments:
-                if isinstance(event, MappingStartEvent) and \
+                if isinstance(event, start_events) and \
                    getattr(comment, 'pre_done', None):
                     continue
                 if self.column != 0:
                     self.write_line_break()
                 self.write_comment(comment)
-                if isinstance(event, MappingStartEvent):
+                if isinstance(event, start_events):
                     comment.pre_done = True
         except TypeError:
             print('eventtt', type(event), event)
